-module(event_mgr).

-behaviour(gen_server).

-export([start_link/1, init/1, handle_cast/2, handle_call/3, handle_info/2]).

-export([add_fields/2]).

-record(event_state, {owner, config = none, fields = [], database = none, supervisor}).

start_link(Args) ->
  gen_server:start_link({local, ?MODULE}, ?MODULE, Args, []).

init([Owner]) ->
  {ok, Supervisor} = event_sup:start_link(),
  {ok, #event_state{owner = Owner, supervisor = Supervisor}}.

get_field_state(_, []) -> free;
get_field_state(Name, [{FieldName, FieldState} | FieldStates]) ->
  if Name == FieldName -> FieldState;
     true -> get_field_state(Name, FieldStates)
  end.

add_fields(Fields, FieldStates) ->
  [{Name, get_field_state(Name, FieldStates)} || Name <- Fields].


% TODO: Calculate which event should be running on this field
% Priority List Logic:
%  1) if testing field, assign testing event
%  2) if skills field, assign skills event
%  3) if division field
%    0) in general, assign match if: (state != done && (num % num_fields) == field_index)
%    1) get practice match from DB
%    2) get qualification match from DB
%    3) get elimination match from DB
%    4) generate/write elimination match to DB
%  4) if finals field
%    1) get finals match from db
%    2) generate/write finals match to DB
%
% If the event requires multiple fields, then we need to check if the other field is free
%  <- does this add the opportunity bad behaviour? Is there a possibility for two events
%     that want to be on fields A & B to try and start at the same time? If so then both events would
%     take both fields(since they only know the state before starting update_field)
%  <- I can deal with this by making update_field 'filter' through the list with both halfs available
%     for each call
update_fields(State, [], Updated) -> 
  State#event_state{fields = Updated};
update_fields(State, [{Name, free} | FieldStates], Updated) ->
  % TODO: assign field logic..
  update_fields(State, FieldStates, [{Name, free} | Updated]);
update_fields(State, [FieldState | FieldStates], Updated) ->
  update_fields(State, FieldStates, [FieldState | Updated]).

update_events(State) ->
  % TODO: calculate which events should be running
  % TODO: stop events on fields that no longer exist
  % TODO: stop events on fields that are running the wrong event
  % TODO: start every event that should be running given the config
  update_fields(State, State#event_state.fields, []).

handle_call(_, _, State) ->
  {noreply, update_events(State)}.

handle_cast(_, State) ->
  {noreply, update_events(State)}.

handle_info(_, State) ->
  {noreply, update_events(State)}.
